package dk.eobjects.metamodel.schema;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import dk.eobjects.metamodel.util.CollectionUtils;

public class CompositeSchema extends AbstractSchema {

	private static final long serialVersionUID = -4018186004438335133L;

	private static final Logger logger = LoggerFactory
			.getLogger(CompositeSchema.class);

	private final String name;
	private final Collection<? extends Schema> delegates;

	public CompositeSchema(String name, Collection<? extends Schema> delegates) {
		this.name = name;
		this.delegates = delegates;
		if (logger.isWarnEnabled()) {
			Set<String> names = new HashSet<String>();
			for (Table table : getTables()) {
				if (names.contains(table.getName())) {
					logger.warn("Name-clash detected for Table {}.",
							table.getName());
					logger.warn(
							"getTableByName(\"{}\") will return just the first table.",
							table.getName());
				} else {
					names.add(table.getName());
				}
			}
			if (!names.isEmpty()) {
				logger.warn("The following table names clashes in composite schema: "
						+ names);
			}
		}
	}

	@Override
	public Relationship[] getRelationships() {
		Relationship[] result = new Relationship[0];
		for (Schema delegate : delegates) {
			result = CollectionUtils.array(result, delegate.getRelationships());
		}
		return result;
	}
	
	@Override
	public int getRelationshipCount() {
		int result = 0;
		for (Schema delegate : delegates) {
			result += delegate.getRelationshipCount();
		}
		return result;
	}

	@Override
	public Table[] getTables() {
		Table[] result = new Table[0];
		for (Schema delegate : delegates) {
			result = CollectionUtils.array(result, delegate.getTables());
		}
		return result;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public String getQuote() {
		return null;
	}
}
